
/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2018 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* $Descr: Purify Library Source for Stack Management (included across TP.c[pp]) $
*******************************************************************************/

/*******************************************************************************
* Global variables
*******************************************************************************/
#if ATP_REPORTS_FIU
#ifdef ATL_ISFILEOPEN
RTRT_GLOBALVAR RTRT_U_INT8 _ata_previously_opened[ATL_STD_IO_MAX_FD-3];
#endif /* ATL_ISFILEOPEN */
#endif /* ATP_REPORTS_FIU */


/*******************************************************************************
* Services:_ata_dump_ptr
*******************************************************************************/
#if RTRT_64_BITS_PTR
RTRT_STATICFCT void _ata_dump_ptr
#if !RTRT_KR
( RTRT_INT_SIZE_PTR v )
#else
( v )
RTRT_INT_SIZE_PTR v;
#endif
{
  atl_line_add_char ( ' ' );
  if(v>>32)
  {
		atl_line_inc ( unsigned_reverse_image_16 ((RTRT_U_INT32)(v>>32),atl_line_cur ) );
		atl_line_add_char ( ',' );
  }
  atl_line_inc ( unsigned_reverse_image_16 ((RTRT_U_INT32)(v&0xFFFFFFFFUL),atl_line_cur ) );
}
#endif /* RTRT_64_BITS_PTR */

/*******************************************************************************
* Services:_ata_dump_uint Dump an unsigned 32bits
*******************************************************************************/
RTRT_STATICFCT void _ata_dump_uint
#if !RTRT_KR
( RTRT_U_INT32 v )
#else
( v )
RTRT_U_INT32 v;
#endif
{
	  atl_line_add_char ( ' ' );
	  atl_line_inc ( unsigned_reverse_image_16 (
	                   v,
	                   atl_line_cur ) );
}
/*******************************************************************************
*  Service: _ata_dump_start_test
*******************************************************************************/
RTRT_STATICFCT void _ata_dump_start_test
#if !RTRT_KR
( void )
#else
( )
#endif
{
  if ( ! _atp_test_started )
  {
#if RTRT_GET_DATE != RTRT_NONE
    char *date;
#endif
    _atp_test_started = 1;
    /* New Test */
    ATL_LINE_ADD_OPCODE ( ATL_TPF_PT );
    _atl_obstools_test_name ( );
#if RTRT_GET_DATE != RTRT_NONE
    date = priv_date();
    if ( date != (char *)0 )
    {
      atl_line_add_char ( ' ' );
      atl_line_add_char ( RTRT_QUOT );
      atl_line_add ( date );
      atl_line_add_char ( RTRT_QUOT );
    }
#endif
    atl_line_dump ( );
  }
}

#if _ATL_SRCNUMBER && !USE_ATT
/*******************************************************************************
*  Service: _ata_dump_source_info
*******************************************************************************/
RTRT_STATICFCT void _ata_dump_source_info
#if !RTRT_KR
( _AtlStackSourceP Source )
#else
( Source )
  _AtlStackSourceP Source;
#endif
{
  /* Print the SF line when ATT is not used */
	  ATL_LINE_ADD_OPCODE ( ATL_TDF_SG );
	  _ata_dump_uint(Source->_SOURCEFILENUMBER);
	  _ata_dump_uint(Source->Key);
	  _ata_dump_uint(Source->CheckSum);
  atl_line_dump ( );
}
#endif /*_ATL_SRCNUMBER && !USE_ATT*/

/*******************************************************************************
*  Service: _ata_dump_stack_info
*******************************************************************************/
RTRT_STATICFCT void _ata_dump_stack_info
#if !RTRT_KR
( _AtlStackInfoP stack_info )
#else
( stack_info )
  _AtlStackInfoP stack_info;
#endif
{
  /* Print the PF line for a routine in the call stack */
#if _ATL_SRCNUMBER
#if !USE_ATT
  if(stack_info->Source->_SOURCEFILENUMBER == 0)
  {
	  stack_info->Source->_SOURCEFILENUMBER = ++_atl_highest_source;
	  _ata_dump_source_info(stack_info->Source);
  }
#endif /*!USE_ATT*/
  ATL_LINE_ADD_OPCODE ( ATL_TPF_PG );
  _ata_dump_uint(stack_info->Source->_SOURCEFILENUMBER);
#else /*_ATL_SRCNUMBER*/
  ATL_LINE_ADD_OPCODE ( ATL_TPF_PF );
  _ata_dump_uint(stack_info->Source->Key);
  _ata_dump_uint(stack_info->Source->CheckSum);
#endif /*_ATL_SRCNUMBER*/
  _ata_dump_uint(stack_info->MessageTsf);
#if _ATL_SRCLNOFFSET
  _ata_dump_uint(stack_info->SourceLineOffset);
#endif /*_ATL_SRCLNOFFSET*/
  atl_line_dump ( );
}

/*******************************************************************************
*  Service: _ata_stack_dump Dump the current call stack
*******************************************************************************/
RTRT_STATICFCT void _ata_stack_dump
#if !RTRT_KR
( _AtlStackLinkP stack_link, int call_stack_size )
#else
( stack_link, call_stack_size)
  _AtlStackLinkP stack_link;
   int call_stack_size ;
#endif
    /* Dump the current call stack */
{
    for ( ;
        call_stack_size-- &&
        stack_link
#if ATL_CHECK_STACK
        && ! _ATL_STACK_CHECK_MAGIC(stack_link)
#endif
        ;
        stack_link = stack_link->Up )
    { /* Dump call stack */
      _ata_dump_stack_info ( &stack_link->Info );
    }
}

/*******************************************************************************
*  Service: _ata_dump_err
*******************************************************************************/
RTRT_STATICFCT void _ata_dump_err
#if !RTRT_KR
(RTRT_CONST_STRING_DECL  err,RTRT_INT_SIZE_PTR v)
#else
( err,v )
  RTRT_CONST_STRING_DECL  err;
  RTRT_INT_SIZE_PTR v;
#endif
{
  _AtlStackLinkP stack_link;

  stack_link = _atl_get_stack_current ( );
  if(stack_link || v)
  {
    _ata_dump_start_test ( );
    /* Dump the Error event line */
    ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PE );
    RTRT_PRINT_ERROR_BEGIN
    atl_line_add ( err );
    if(v) ata_dump_ptr(v);
    RTRT_PRINT_ERROR
    atl_line_dump ( );

    /* Dump the current call stack */
    _ata_stack_dump(stack_link,ATP_CALL_STACK_SIZE);
  }
}

#if ATP_REPORTS_FIU
#ifdef ATL_ISFILEOPEN
/*******************************************************************************
*  Service: _ata_dump_FIU (File Descriptor in Use)
*******************************************************************************/
RTRT_STATICFCT void _ata_dump_FIU
#if !RTRT_KR
( void )
#else
( )
#endif
{
  int i,atl_out_fd;
  char status;

  /* Dump the FIU event line */
  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PE );
  ATL_LINE_ADD_OPCODE ( "FIU" );
  atl_line_dump ( );
  atl_out_fd=0;
  if((atl_out) && (atl_out !=(RTRT_FILE_TYPE)(1)))
      atl_out_fd=ATL_FILENO ( atl_out );

  /* Parse file descriptors up to ATL_STD_IO_MAX_FD */
  for ( i = 3; i < ATL_STD_IO_MAX_FD; i++ )
  {
    if ( i == atl_out_fd )
    {
      status = '0'; /* internal use */
    }
    else if ( ATL_ISFILEOPEN ( i ) )
    {
      if ( _ata_previously_opened[i-3] )
      {
        continue; /* Yet opened (Parent process inherited) */
      }
      status = '1'; /* Locally opened and still not closed */
    }
    else
    {
      continue;
    }
    ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PO );
    atl_line_inc ( unsigned_reverse_image_16 ( i, atl_line_cur ) );
    atl_line_add_char ( ' ' );
    atl_line_add_char ( status );
    atl_line_dump ( );
  }
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}
#endif /* ATL_ISFILEOPEN */
#endif /* ATP_REPORTS_FIU */

#if RTRT_CPLUSPLUS_LANGUAGE && !RTRT_STATIC_AND_EXTERN_C_FORBIDDEN
extern "C" {
#endif

/*******************************************************************************
*  Service: ata_dump_CALLSTACK for #pragma attol atp_insert _ATL_DUMP_STACK("signalName")
*******************************************************************************/
_ATL_C_GLOBALFCT void ata_dump_CALLSTACK
#if !RTRT_KR
(const char *name, _AtlStackLinkP stack_link)
#else
( name , stack_link )
  char * name;
  _AtlStackLinkP stack_link;
#endif
{
  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  _ata_dump_start_test ( );
  stack_link = _atl_get_stack_current ( );
  /* Dump the SIG event line */
  ATL_LINE_ADD_OPCODE ( "PE SIG 0 \"" );
  if(name) atl_line_add      ( name );
  atl_line_add_char ( RTRT_QUOT );

#if ATL_MULTI_THREADS
  _ata_dump_uint((RTRT_U_INT32)(_atl_threads_table_used[stack_link->ThreadIndex]));
#endif
  atl_line_dump ( );

  /* Dump the complete current call stack */
  _ata_stack_dump(stack_link,-1);

#if ATL_MULTI_THREADS && defined(DUMP_STACK_ALL)
 {
  int i;
  for (i = 0; i < ATL_THREADS_MAX; i++) {
    stack_link = _atl_stack_table_current[i];
    /* Dump the call stack */
    if (RTRT_MT_THREAD_ISALIVE(_atl_threads_table_used[i])) {
      ATL_LINE_ADD_OPCODE ( "PE SIG 0 \"" );
      if(name) atl_line_add      ( name );
      atl_line_add_char ( RTRT_QUOT );
      _ata_dump_uint((RTRT_U_INT32)(_atl_threads_table_used[stack_link->ThreadIndex]));
      _ata_dump_uint(i);
      atl_line_dump ( );
      _ata_stack_dump(stack_link,-1);
    }
  }
 }
#endif
  ATL_BUFFER_FLUSH ( );
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}

#if USE_ATA
#if ATP_REPORTS_SIG
#if RTRT_SIGNAL == RTRT_STD

/*******************************************************************************
*  Service: _ata_dump_SIG (Signal handled)
*******************************************************************************/
RTRT_STATICFCT void ATL_CALLCONV _ata_dump_SIG
#if !RTRT_KR
( int signum )
#else
( signum )
  int signum;
#endif
{
  int i;
  char *str;
  _AtlStackLinkP stack_link;

  /* Restore previous handler */
  for ( i = 0;
        i < ATL_SIGNALS_CATCHED_CARD &&
        _atl_signal_datas[i].SigNum == signum;
        i++ );

  if ( i != ATL_SIGNALS_CATCHED_CARD )
  { /* Strange if not found but... */
    ATL_SIGNAL ( signum, _atl_signal_datas[i].Handler );
  }
  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  _ata_dump_start_test ( );
  /* Dump the SIG event line */
  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PE );
  RTRT_PRINT_ERROR_BEGIN
  ATL_LINE_ADD_OPCODE ( "SIG " );
  atl_line_inc ( unsigned_reverse_image_16 ( (RTRT_U_INT32)signum,
                                             atl_line_cur ) );
  RTRT_PRINT_ERROR
  str = _atl_signal_string ( signum );
  if ( str != (char *)0 )
  {
    ATL_LINE_ADD_OPCODE ( " \"" );
    atl_line_add      ( str );
    atl_line_add_char ( RTRT_QUOT );
  }
  atl_line_dump ( );

  stack_link = _atl_get_stack_current ( );

  /* Dump the current call stack */
  _ata_stack_dump(stack_link,ATP_CALL_STACK_SIZE);

  ATL_BUFFER_FLUSH ( );
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)

#ifdef RTRT_POST_SIG
  RTRT_POST_SIG
#endif /* RTRT_POST_SIG */

  /* and raise again */
  ATL_RAISE ( signum );
}

#endif /* RTRT_SIGNAL == RTRT_STD */
#endif /* ATP_REPORTS_SIG */
#endif /* USE_ATA */

#if RTRT_CPLUSPLUS_LANGUAGE && !RTRT_STATIC_AND_EXTERN_C_FORBIDDEN
} /* end of extern "C" */
#endif

#if !USE_ATP
/*******************************************************************************
*  Service: _ata_dump  not activated USE_ATP. this code is already in atp_dump
*******************************************************************************/
_ATL_C_GLOBALFCT void _ata_dump
#if !RTRT_KR
( int reset )
#else
( reset )
  int reset;
#endif
{
  if ( reset & _ATCPQ_PRF)
  {
#if USE_ATA && ATP_REPORTS_FIU
#ifdef ATL_ISFILEOPEN
	TDPDEBUG("report FIU")
  	_ata_dump_FIU ( );
#endif /* ATL_ISFILEOPEN */
#endif /* ATP_REPORTS_FIU */

  /* Reset for next test */
  	_atp_test_started = 0;
  }
}
#endif /*!USE_ATP */

/*******************************************************************************
*  Service: atp_early_init
*  Goal: initializes global variables for FIU and SIG
*  and generate report header for .tpf file
*******************************************************************************/
_ATL_C_GLOBALFCT void ata_early_init
#if !RTRT_KR
( void )
#else
( )
#endif
{
#if USE_ATP || (USE_ATZ && ATP_STACK_SIZE_LEVEL) || (USE_ATA && (ATP_REPORTS_FIU || ATP_REPORTS_SIG))
  /* Writes heading codes */
  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PU );
  atl_line_add_char ( RTRT_QUOT );
  atl_buffer_add (ATL_NODE_NAME);
  atl_line_add_char ( RTRT_QUOT );
  atl_line_dump ( );
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)

  _atp_test_started = 0;
#endif /*USE_ATP || (USE_ATZ && ATP_STACK_SIZE_LEVEL) || (USE_ATA && (ATP_REPORTS_FIU || ATP_REPORTS_SIG))*/

#if USE_ATA
#if ATP_REPORTS_FIU
#ifdef ATL_ISFILEOPEN
  { int i;
	  for ( i = 3; i < ATL_STD_IO_MAX_FD; i++ )
	  {
		_ata_previously_opened[i-3] = (RTRT_U_INT8)ATL_ISFILEOPEN(i);
	  }
  }
#endif /* ATL_ISFILEOPEN */
#endif /* ATP_REPORTS_FIU */

#if ATP_REPORTS_SIG
#if RTRT_SIGNAL == RTRT_STD
  _atl_signal_catch ( _ata_dump_SIG );
#endif /* RTRT_SIGNAL == RTRT_STD */
#endif /* ATP_REPORTS_SIG */
#endif /* USE_ATA */
}

