

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2011 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* $Descr: Stack Size Library Source (included by TP.c[pp]) $
*******************************************************************************/


/*******************************************************************************
* Global variables
*******************************************************************************/

/* Header of linked-list */
RTRT_GLOBALVAR _atz_TP _atz_first;
RTRT_STATICVAR char _atz_test_started;
/* What is the message corresponding to an index */
#define _ATZ_MESSAGE_FROM_INDEX(i) \
  ((((i)+1)&0x0000001f)|((((i)+1)&0xffffffe0)<<1))

/* What is the index corresponding to a message */
#define _ATZ_INDEX_FROM_MESSAGE(m) \
  ((((m)&0x0000001f)|(((m)&0xffffffc0)>>1))-1)

/*******************************************************************************
*  Service: _atz_dump
*******************************************************************************/
_ATL_C_GLOBALFCT void _atz_dump
#if !RTRT_KR
( int reset )
#else
( reset )
  int reset;
#endif
{
  _atz_TP thedata;
  RTRT_U_INT32 i;
  RTRT_U_INT8 printHead;

  if ( reset & _ATCPQ_STACK)
  {

	  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
	  if(!_atz_test_started)
	  {
		  ATL_LINE_ADD_OPCODE ( ATL_TPF_ZD );
		  atl_line_dump ( );
		  _atz_test_started=1;
	  }

	  ATL_LINE_ADD_OPCODE ( ATL_TPF_ZT );
	  _atl_obstools_test_name ( );
	  _ata_dump_uint(sizeof(_AtlStackLink));
	  atl_line_dump ( );
	  
	  for ( thedata = _atz_first;
			thedata;
			thedata = thedata->Next )
	  {
		printHead=0;
		for ( i = 0; i < thedata->Routines; i++ )
		{
		  if ( thedata->StackSizes[i] )
		  {
			if(printHead==0)
			{
				ATL_LINE_ADD_OPCODE ( ATL_TPF_ZF );
#if ATL_DYN_TABLE
				_ata_dump_uint( thedata->Key);
				_ata_dump_uint( thedata->CheckSum);
#else /* ATL_DYN_TABLE */
				_ata_dump_uint(thedata->Source->Key);
				_ata_dump_uint(thedata->Source->CheckSum);
#endif /* ATL_DYN_TABLE */
				atl_line_dump ( );
				printHead=1;
			}
			ATL_LINE_ADD_OPCODE ( ATL_TPF_ZM );
			_ata_dump_uint(_ATZ_MESSAGE_FROM_INDEX(i));
			_ata_dump_uint(thedata->StackSizes[i]);
			atl_line_dump ( );
			if(reset & _ATCPQ_RESET_STACK)
				thedata->StackSizes[i]=0;
	  	  }
	    }
	  }
	  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
  }
}
/*******************************************************************************
*  remove data for a source Family
*******************************************************************************/
#ifdef ATL_T_FAMILY
void _atz_remove_family
#if !RTRT_KR
(ATL_T_FAMILY family)
#else
(family)
ATL_T_FAMILY family;
#endif
{
#if !ATL_DYN_TABLE

  _atz_TP thedata,dataPrev;
   thedata = _atz_first;
    while((thedata) && (thedata->Source->Family == family))
    {
		thedata = thedata->Next;
	}
	_atz_first = thedata;
    dataPrev = thedata;
	while(thedata)
	{
       if(thedata->Source->Family == family)
       {
	 	 dataPrev->Next = thedata->Next;
	   }
	   else
	   {
		   dataPrev = thedata;
	   }
	   thedata = thedata->Next;
	}
#endif /* ATL_DYN_TABLE */
}
#endif

/*******************************************************************************
*  Service: _atz_start
*******************************************************************************/
_ATL_C_GLOBALFCT void _atz_start
#if !RTRT_KR
( _AtlStackLinkP stack_link )
#else
( stack_link )
  _AtlStackLinkP stack_link;
#endif
{
  _atz_TP thedata;

  ATZ_THREAD_FILTERING
#if ATL_CHECK_STACK
  if ( _ATL_STACK_CHECK_MAGIC(stack_link) )
  {
    ATL_ERROR ( 'Q',"_atz_start(): runtime analysis stack data is corrupted" );
    return;
  }
#endif /* ATL_CHECK_STACK */
  thedata = (_atz_TP)(stack_link->AtzData);
#if ATL_DYN_TABLE
  if ( ! thedata->Key )
  {
    RTRT_U_INT32 i;

    thedata->Key = stack_link->Info.Source->Key;
    thedata->CheckSum = stack_link->Info.Source->CheckSum;
#else /* ATL_DYN_TABLE */
  if ( ! thedata->Source )
  {
    RTRT_U_INT32 i;

    thedata->Source = stack_link->Info.Source;
#endif /* ATL_DYN_TABLE */
    for ( i = 0; i < thedata->Routines; i++ )
    {
      thedata->StackSizes[i] = (RTRT_SP_SIZE_VAR_T)0;
    }
    thedata->Next = _atz_first;
    _atz_first = thedata;
  }
  /* recover previous measure */
  stack_link->Stack_size =thedata->StackSizes[_ATZ_INDEX_FROM_MESSAGE(stack_link->Info.MessageTsf)];
  _atl_sp_size_measure(&stack_link);

}

/*******************************************************************************
*  Service: atz_set
*  Goal: set source local var
*******************************************************************************/
_ATL_C_GLOBALFCT void atz_set
#if !RTRT_KR
( _AtlStackLinkP stack_link, RTRT_SP_SIZE_VAR_T size)
#else
( stack_link, size )
  _AtlStackLinkP stack_link;
  RTRT_SP_SIZE_VAR_T size
#endif
{
	if(stack_link->AtzData) /* to be safe */
	{
		RTRT_SP_SIZE_VAR_T *p=&(((_atz_TP)(stack_link->AtzData))->StackSizes[_ATZ_INDEX_FROM_MESSAGE(stack_link->Info.MessageTsf)]);
		if(*p<size) *p=size;
	}
}

/*******************************************************************************
*  Service: _atl_sp_size_measure
*  This method compute the SP size value and keep the max into static buffer of the instrumented file
*******************************************************************************/
RTRT_STATICFCT void _atl_sp_size_measure(_AtlStackLinkP * slp)
{
  RTRT_INT32 sizeMethod;
  void * curSP;
  void * start_sp;
  _AtlStackLinkP stack_link=*slp;
#if ATL_CHECK_STACK
  if ( stack_link && _ATL_STACK_CHECK_MAGIC(stack_link) )
  {
    ATL_ERROR ('K', "_atl_sp_size_measure(): runtime analysis stack data is corrupted" );
    return;
  }
#endif /* ATL_CHECK_STACK */

	/* read parameter address => stack pointer value */
	curSP =(void*) slp;

	if(stack_link->Up)
	{
		start_sp=stack_link->Up->Stack_spref;
	}
	else
	{
#if USE_ATU || USE_ATV
		start_sp=_atz_ref_test_start;
#else /* USE_ATU */
		if(_atl_stack_direction==2)
		{
		start_sp=(void*)stack_link;
		}
		else
		{
			/* must include the AtlStackLink size */
			_AtlStackLinkP slpl=stack_link;
			slpl++;
			start_sp=(void*)slpl;
		}
#endif /* USE_ATU */
	}
	/* stack actual size = start_sp - actual stack pointer (&parameter) */
	sizeMethod =(RTRT_INT32)(RTRT_PTR_TO_INT(start_sp) - RTRT_PTR_TO_INT(curSP));
		
	/* correct stack management UP/Down */
	if(_atl_stack_direction==2)
	{
		sizeMethod = -sizeMethod;
	}
	/* keep the biggest in local */
	if(sizeMethod<=(RTRT_INT32)0)
		sizeMethod=sizeof(void*);/* at least return address */ 
		if(stack_link->Stack_size < sizeMethod)
			stack_link->Stack_size=sizeMethod;
}

/*******************************************************************************
*  Service: _atl_stack_measure called before call to an other method to re-set the reference for this call
*  This method compute the SP size value and keep the max into static buffer of the instrumented file
*******************************************************************************/
_ATL_C_GLOBALFCT void _atl_stack_measure
#if !RTRT_KR
( _AtlStackLinkP stack_link )
#else
( stack_link )
  _AtlStackLinkP stack_link;
#endif
{
	stack_link->Stack_spref=&stack_link;
	
#if ATA_STACK_MULTIPLE
	_atl_sp_size_measure(&stack_link);
#endif /*ATA_STACK_MULTIPLE*/
}

/*******************************************************************************
*  Goal: initializes _atl_stack_direction
*******************************************************************************/
RTRT_STATICFCT void upDown2(void *p,void **px)
{
        void **x=&p;
        if(x<px)
                _atl_stack_direction=1; /*stack pointer goes up(decrease)*/
        else
                _atl_stack_direction=2;
}
RTRT_STATICFCT void upDown(void *p)
{
        upDown2((void *)0,&p);
}

/*******************************************************************************
*  Service: atz_early_init
*  Goal: initializes global variables
*******************************************************************************/
_ATL_C_GLOBALFCT void atz_early_init
#if !RTRT_KR
( void )
#else
( )
#endif
{
#if ! RTRT_RAM_CLEAN
  _atz_first = (_atz_TP)0;
  _atz_test_started=0;
#endif
  upDown((void*)0);
}

/*******************************************************************************
*  Service: atz_late_exit
*  Goal: called when it is time to quit
*******************************************************************************/
_ATL_C_GLOBALFCT void atz_late_exit
#if !RTRT_KR
( void )
#else
( )
#endif
{
#if ! USE_ATU && ! USE_ATO && ! USE_ATV
  _atz_dump ( _ATCPQ_ALL );
#endif /* USE_ATU && USE_ATO */
}
