
/************************************************************************
* Licensed Materials - Property of IBM and/or HCL
* (c) Copyright IBM Corporation 2003, 2016. All Rights Reserved.
* (c) Copyright HCL Technologies Ltd. 2017.  All Rights Reserved.
*
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp.
*************************************************************************/




/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2011 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* $Descr: Common buffer library (included by TP.c[pp]) $
*******************************************************************************/


/*******************************************************************************
* Static variables
*******************************************************************************/

/*! Buffer where lines are built */
RTRT_GLOBALVAR char atl_buffer[ATL_BUFFER_SIZE];

#ifdef RTRT_LINE_SPLIT_MARKER_BEGIN
/*! Pointer to the first useful character in buffer */
RTRT_STATICVAR char *atl_buffer_top;
#else
#define atl_buffer_top atl_buffer
#endif

#ifdef RTRT_PRINT_ERRORS
RTRT_STATICVAR char *rtrt_print_error_string;
#define RTRT_PRINT_ERROR 		ATL_ERROR ('_',rtrt_print_error_string);
#define RTRT_PRINT_ERROR_BEGIN  rtrt_print_error_string=atl_buffer_next;
#endif

#ifndef RTRT_PRINT_ERROR
#define RTRT_PRINT_ERROR_BEGIN
#define RTRT_PRINT_ERROR
#endif

/*! Pointer to the next character in buffer */
RTRT_STATICVAR char *atl_buffer_next;

/*! Pointer to the first character in buffer slack */
RTRT_STATICVAR char *atl_buffer_slack;

/*! Mutex variable controlling access to atl_buffer */
RTRT_MT_CRITICAL_VAR_DECL(atl_buffer_mutex)

#if ATL_MULTI_PROCESS
RTRT_STATICVAR RTRT_U_INT32 _atl_current_pid;
#endif /* ATL_MULTI_PROCESS */

/*! Macro to get remaining space in buffer */
#define _ATL_BUFFER_REMAINING (atl_buffer_slack-atl_buffer_next)
/*! Macro allowing newline char modification by the TDP */
#ifndef RTRT_NEWLINE_CHAR
#define RTRT_NEWLINE_CHAR '\n'
#endif /* RTRT_NEWLINE_CHAR */

#ifdef RTRT_INT64
#define _MAX_DIGIT 21
#else
#define _MAX_DIGIT 11
#endif

#if (USE_ATO || USE_ATV || USE_ATU) && RTRT_FLOAT
#define _MAX_FLT_DIGIT   RTRT_NBDIGITS_LDBL
#if     _MAX_FLT_DIGIT < RTRT_NBDIGITS_DBL
#undef  _MAX_FLT_DIGIT
#define _MAX_FLT_DIGIT   RTRT_NBDIGITS_DBL
#endif
#if     _MAX_FLT_DIGIT < RTRT_NBDIGITS_FLT
#undef  _MAX_FLT_DIGIT
#define _MAX_FLT_DIGIT   RTRT_NBDIGITS_FLT
#endif
/* 3: sign+unit+dot, 5:e+sign+exp 3 for DEN = 11 */
#if     _MAX_DIGIT < (_MAX_FLT_DIGIT+11)
#define _MAX_DIGIT_NUM   (_MAX_FLT_DIGIT+11)
#else
#define _MAX_DIGIT_NUM   _MAX_DIGIT
#endif
#else
#define _MAX_DIGIT_NUM   _MAX_DIGIT
#endif /* (USE_ATO || USE_ATV || USE_ATU) && RTRT_FLOAT */

/* #if (ATL_BUFFER_SIZE <256) || ((USE_ATO || USE_ATV || USE_ATU) && RTRT_FLOAT) */
/* reduce if the slack takes more than 10% of the buffer */
#if ATL_BUFFER_SIZE < (5*_MAX_DIGIT_NUM)
#define ATL_CHECK_SLACK
#if ATL_BUFFER_SIZE < _MAX_DIGIT_NUM
#define _ATL_BUFFER_SLACK_SIZE 8
#else  /* ATL_BUFFER_SIZE < _MAX_DIGIT_NUM */
#define _ATL_BUFFER_SLACK_SIZE _MAX_DIGIT_NUM
#endif /* ATL_BUFFER_SIZE < _MAX_DIGIT_NUM */
#else
/* on complete line should take place in the slack */
#define _ATL_BUFFER_SLACK_SIZE ((3 * _MAX_DIGIT_NUM) + 18 )
#endif

#if RTRT_IO == RTRT_NONE
/* do not break the execution for few lines */
#define  ATL_BUFFER_DUMP()
#define  ATL_BUFFER_FLUSH()
#else /* */
#define  ATL_BUFFER_DUMP() atl_buffer_dump()
#define  ATL_BUFFER_FLUSH() if(atl_out) atl_buffer_dump()
#endif

RTRT_STATICFCT void atl_buffer_add
#if !RTRT_KR
  (RTRT_CONST_STRING_DECL str)
#else
   ()
#endif
;
/*******************************************************************************
* Static functions (or macros)
*******************************************************************************/
#define atl_line_add_char(c) *atl_buffer_next++ = c
#if RTRT_IO == RTRT_NONE
#define atl_buffer_write priv_writeln
#endif

/* postGen can change following line as compile or linker instruction to keep the symbol despite the optimizer */
/*#pragma KEEP_FUNC priv_close */
_ATL_C_GLOBALFCT void atl_buffer_write
#if !RTRT_KR
  (void)
#else
  ()
#endif
  {
#if RTRT_IO == RTRT_NONE
#ifdef PRIV_WRITELN_HEAD_CODE
	   PRIV_WRITELN_HEAD_CODE
#endif /*PRIV_WRITELN_HEAD_CODE*/
#ifdef NEEDS_atl_stop
	if(atl_stop>1) atl_stop=0; /* make sure optimiser does not remove this function */
#endif
	RTRT_WATCH_DOG
#else
	priv_writeln  (atl_out, atl_buffer);
#endif
	atl_buffer_next = atl_buffer;
	*atl_buffer_next = '\0';
#ifdef RTRT_LINE_SPLIT_MARKER_BEGIN
	atl_buffer_add(RTRT_LINE_SPLIT_MARKER_BEGIN);
	atl_buffer_top = atl_buffer_next;
#endif
#ifdef ATL_CHECK_SLACK
	/* check overwrite */
	atl_buffer[ATL_BUFFER_SIZE-1]=(char)0xFF;
#endif
  }

/*! Dump the buffer into the output */
RTRT_STATICFCT void atl_buffer_dump
#if !RTRT_KR
  (void)
#else
  ()
#endif
{
#ifdef RTRT_LINE_SPLIT_MARKER_END
  char *buffer_slack_sav;
#endif
#ifdef ATL_CHECK_SLACK
  char lastBuffChar;
#endif
  if (atl_buffer_next == atl_buffer_top)
  {
    return;
  }
#ifdef RTRT_LINE_SPLIT_MARKER_END
  buffer_slack_sav=atl_buffer_slack;
  atl_buffer_slack=atl_buffer + ATL_BUFFER_SIZE-1;
  if(atl_buffer_next < atl_buffer_slack)
  {
    atl_buffer_add(RTRT_LINE_SPLIT_MARKER_END);
  }
  atl_buffer_slack = buffer_slack_sav;
#endif
#ifdef ATL_CHECK_SLACK
  /* check overwrite */
  lastBuffChar=atl_buffer[ATL_BUFFER_SIZE-1];
#endif
  *atl_buffer_next = '\0';
#ifdef NEEDS_atl_initialized
  if (atl_initialized > 3)
  {
#if _ATL_ERROR_IMPLEMENTED /* avoid reentrance */
#if !_ATL_TRACE_MAY_OCCUR_AFTER_EXIT
    ATL_ERROR ('Z',"attempt to use Target Deployment Port resources after exit handler has been called (level 4)");
#endif
#endif
    atl_buffer_next = atl_buffer;
    return;
  }
#endif /* NEEDS_atl_initialized */
#if RTRT_IO != RTRT_NONE
  if (atl_out == (RTRT_FILE_TYPE)0
#if ATL_MULTI_PROCESS
     || _atl_current_pid != priv_getpid()
#endif /* ATL_MULTI_PROCESS */
     )
  {
    atl_out_open();
  }
#endif /* RTRT_IO == RTRT_NONE */
  atl_buffer_write();
#ifdef ATL_CHECK_SLACK
  /* check overwrite */
  if(lastBuffChar!=(char)0xFF)
  {
	 atl_line_add_char(0x0A);
	 atl_line_add_char('W');
	 atl_line_add_char('A');
	 atl_line_add_char(' ');
	 atl_line_add_char(0x29);
	 atl_line_add_char(0x0A);
  }
#endif
}

RTRT_STATICFCT void atl_buffer_dump_if
#if !RTRT_KR
  (void)
#else
  ()
#endif
{
	if(atl_buffer_next >= atl_buffer_slack)
		atl_buffer_dump();
}
/*! Append string str to the buffer
 * check buffer limit at the beginning and the end and evry slack_size
 */
RTRT_STATICFCT void atl_buffer_add
#if !RTRT_KR
  (RTRT_CONST_STRING_DECL str)
#else
   (str)
  RTRT_CONST_STRING_DECL str;
#endif
{
  char i=0;
  while (*str)
  {
    *atl_buffer_next++ = *str++;
    if (i==0 || *str==0)
    {
      atl_buffer_dump_if ();
      i=_ATL_BUFFER_SLACK_SIZE;
    }
    i--;
  }
}

#ifdef ATL_CHECK_SLACK
#define atl_line_add(s) atl_buffer_add(s)
#else
RTRT_STATICFCT void atl_line_add
#if !RTRT_KR
  (RTRT_CONST_STRING_DECL str)
#else
   (str)
  RTRT_CONST_STRING_DECL str;
#endif
{
  while (*str)
  {
    *atl_buffer_next++ = *str++;
  }
}
#endif

#if !defined( ATL_CHECK_SLACK ) && !defined(ATL_NOADDPTR)
#define atl_line_inc(l) atl_buffer_next += (l)
#else
#define atl_line_inc(l) atl_buffer_inc(l)

/*! Increment the buffer pointer for l characters */
RTRT_STATICFCT void atl_buffer_inc
#if !RTRT_KR
  (int l)
#else
  (l)
  int l;
#endif
{
#ifdef ATL_NOADDPTR
  while(l--) atl_buffer_next++;
#else
  atl_buffer_next += l;
#endif
  atl_buffer_dump_if ();
}
#endif

/*! End current line of the buffer */
RTRT_STATICFCT void atl_buffer_endline
#if !RTRT_KR
  (void)
#else
  ()
#endif
{
  *atl_buffer_next++ = RTRT_NEWLINE_CHAR;
#ifdef _ATL_EXIT_PENDING
  if (_ATL_EXIT_PENDING)
  {
    atl_buffer_dump ();
  }
  else
#endif
    atl_buffer_dump_if ();
}

/*! Initialize the buffer */
RTRT_STATICFCT void atl_buffer_init
#if !RTRT_KR
  (void)
#else
  ()
#endif
{
  atl_buffer_next = atl_buffer;
  atl_buffer_slack = atl_buffer + ATL_BUFFER_SIZE - _ATL_BUFFER_SLACK_SIZE;
  RTRT_MT_CRITICAL_VAR_INIT(atl_buffer_mutex)
#ifdef RTRT_LINE_SPLIT_MARKER_BEGIN
  RTRT_MT_CRITICAL_VAR_LOCK(atl_buffer_mutex)
  atl_buffer_add(RTRT_LINE_SPLIT_MARKER_BEGIN);
  atl_buffer_top = atl_buffer_next;
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_buffer_mutex)
#endif
#if ATL_MULTI_PROCESS
  _atl_current_pid = priv_getpid();
#endif /* ATL_MULTI_PROCESS */
#ifdef ATL_CHECK_SLACK
  /* check overwrite */
  atl_buffer[ATL_BUFFER_SIZE-1]=(char)0xFF;
#endif
}

/*! Dump the remaining buffer and destroy allocated resources. */
RTRT_STATICFCT void atl_buffer_terminate
#if !RTRT_KR
  (void)
#else
  ()
#endif
{
  atl_buffer_dump ();
  *atl_buffer_next = '\0'; /* make sure the usr_close won't print the last buffer twice */
  RTRT_MT_CRITICAL_VAR_DESTROY(atl_buffer_mutex)
}
#if USE_ATU
/*******************************************************************************
*  Service: _atl_add_error used by autostubing to generate error
*		the code is like "W2 " msg is the text
*******************************************************************************/
_ATL_C_GLOBALFCT void _atl_add_error
#if !RTRT_KR
( RTRT_CONST_STRING_DECL errcode,RTRT_CONST_STRING_DECL msg )
#else
( errcode , msg )
  RTRT_CONST_STRING_DECL errcode;
  RTRT_CONST_STRING_DECL msg;
#endif
{
  RTRT_MT_CRITICAL_VAR_LOCK(atl_buffer_mutex)
  atl_line_add ( errcode );
  atl_line_add ( msg  );
  atl_buffer_endline(); /* Add a \n */
  atl_buffer_dump (); /* dump */
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_buffer_mutex)
}
#endif
/*******************************************************************************
*  Service: _atl_error_dump
*******************************************************************************/
_ATL_C_GLOBALFCT void _atl_error_dump
#if !RTRT_KR
( const char icode )
#else
( icode )
  const char icode;
#endif
{
  RTRT_MT_CRITICAL_VAR_LOCK(atl_buffer_mutex)
  ATL_LINE_ADD_OPCODE ( "WA " );
/*  atl_line_inc ( unsigned_reverse_image_16 ( icode, atl_buffer_next ) ); */
  atl_line_add_char(icode);
  atl_buffer_endline(); /* Add a \n */
  atl_buffer_dump (); /* dump */
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_buffer_mutex)
}


#define atl_line_dump atl_buffer_endline
#define atl_line_cur atl_buffer_next
#define atl_line_mutex atl_buffer_mutex
#define _ATL_LINE_REMAINING _ATL_BUFFER_REMAINING
